@@ -43,7 +43,7 @@ elif settings.SESSION_TYPE == "redis":
     host, port = settings.REDIS_SERVER.split(":")
     # for more options: https://github.com/andymccurdy/redis-py/blob/master/redis/client.py
     conn = redis.Redis(host=host, port=int(port))
-    conn.set = lambda k, v, e, cs=conn.set, ct=conn.ttl: (cs(k, v), e and ct(e))
+    conn.set = lambda k, v, e, cs=conn.set, ct=conn.ttl: cs(k, v, ct(k)) if ct(k) >= 0 else cs(k, v, e) 
     session = Session(secret=settings.SESSION_SECRET_KEY, storage=conn)
 elif settings.SESSION_TYPE == "memcache":
     import memcache, time
@@ -97,15 +97,18 @@ if settings.OAUTH2FACEBOOK_CLIENT_ID:
         OAuth2Facebook(
             client_id=settings.OAUTH2FACEBOOK_CLIENT_ID,
             client_secret=settings.OAUTH2FACEBOOK_CLIENT_SECRET,
-            callback_url="auth/plugin/oauth2google/callback",
+            callback_url="auth/plugin/oauth2facebook/callback",
         )
     )
 
 if settings.USE_CELERY:
     from celery import Celery
+
     # to use from . common import scheduled and then use it accoding to celery docs
     # examples in tasks.py
-    scheduler = Celery('apps.%s.tasks' % settings.APP_NAME, broker=settings.CELERY_BROKER)
+    scheduler = Celery(
+        "apps.%s.tasks" % settings.APP_NAME, broker=settings.CELERY_BROKER
+    )
 
 
 # we enable auth, which requres sessions, T, db and we make T available to
